@@ -4,25 +4,28 @@ use strict;
 use vars qw( $VERSION );
 
 #
-# $Id: toaster_setup.pl,v 4.1 2004/11/16 21:20:01 matt Exp $
+# $Id: toaster_setup.pl,v 4.20 2005/04/17 12:52:10 matt Exp $
 #
 
-$VERSION = "4.00";
+$VERSION = "4.04";
 
 =head1 NAME
 
-toaster_setup.pl - Everything you need to build a mail toaster except a computer
+toaster_setup.pl
 
 =head1 SYNOPSIS
 
-To build a great mail system, install FreeBSD (latest stable), and follow
-the directions on the toaster page (see URL below).
+Everything you need to turn a computer into a secure, full-featured, high-performance mail server.
 
 =head1 DESCRIPTION
 
-A complete set of instructions for building a mail toaster are on the toaster install page. There is actually quite a bit of documenation available for the "Matt Style" toaster. Much of it is readable via "perldoc Mail::Toaster", and all the subsequent pages. Don't forget to read the Install, Configure, and FAQ pages on the web site. If you still have questions, the mailing list archives are browseable and searchable for your convenience. 
+The mail toaster is a collection of open-source software which provides a full-featured mail server running on FreeBSD and MacOS X. The system is built around the qmail mail transport agent, with many additions and modifications. Matt Simerson is the primary author and maintainer of the toaster. There is an active and friendly community of toaster owners which supports the toaster on a mailing list and web forum.
 
-	http://www.tnpi.biz/internet/mail/toaster/
+The toaster is built around qmail, a robust mail transfer agent by Daniel J. Bernstein, and vpopmail, a virtual domain manager by Inter7 systems. Matt keeps up with releases of the core software, evaluates them, decides when they are stable, and then integrates them into the toaster. Matt has also added several patches which add functionality to these core programs.
+
+A complete set of instructions for building a mail toaster are on the toaster install page. There is a substantial amount of documenation available for the "Mail::Toaster" toaster. Much of it is also readable via "perldoc Mail::Toaster", and the subsequent pages. Don't forget to read the Install, Configure, and FAQ pages on the web site. If you still have questions, there is a Web Forum and mailing list. Both are browseable and searchable for your convenience. 
+
+  http://www.tnpi.biz/internet/mail/toaster/
 
 =cut
 
@@ -37,9 +40,6 @@ use Mail::Toaster::Apache  4; my $apache  = new Mail::Toaster::Apache;
 use vars qw/ $conf /;
 
 $| = 1;
-my $user = (getpwuid ($<))[0];
-
-if ( $user ne "root") { die "Thou shalt have root to proceed!\n"; };
 
 my %options = (
 	'action=s'    => \my $action,
@@ -48,6 +48,9 @@ my %options = (
 );
 GetOptions (%options);
 
+my $user = (getpwuid ($<))[0];
+if ( $user ne "root" && $section ne "test2" ) { die "Thou shalt have root to proceed!\n"; };
+
 =head2 command line flags
 
 toaster_setup.pl can be passed several flags. 
@@ -75,17 +78,24 @@ $conf = $utility->parse_config( { file=>"toaster-watcher.conf", debug=>$debug} )
 
 $conf->{'debug'} = 1 if $debug;
 
-my $src = $conf->{'toaster_src_dir'};
-unless ( $src ) { $src = "/usr/local/src"; };
+my $src = $conf->{'toaster_src_dir'} || "/usr/local/src";
 
 unless ( $section eq "ports" or $section eq "sources" ) {
 	$perl->check();
 };
 
+
 if    ( $section eq "pre"         ) { $setup->dependencies($conf)          }
 elsif ( $section eq "perl"        ) { $perl->check($conf, $debug)          }
+
+#  FreeBSD specific
 elsif ( $section eq "ports"       ) { $setup->ports($conf)                 }
 elsif ( $section eq "sources"     ) { $freebsd->source_update($conf)       }
+elsif ( $section eq "jailadd"     ) { $freebsd->jail_create()              }
+elsif ( $section eq "jaildelete"  ) { $freebsd->jail_delete()              }
+elsif ( $section eq "jailstart"   ) { $freebsd->jail_start()               }
+
+#  Standard daemons & utilities
 elsif ( $section eq "mysql"       ) { $setup->mysqld($conf)                }
 elsif ( $section eq "phpmyadmin"  ) { $setup->phpmyadmin($conf)            }
 elsif ( $section eq "apache"      ) { $setup->apache($conf,undef)          }
@@ -93,37 +103,63 @@ elsif ( $section eq "apache1"     ) { $setup->apache($conf,1)              }
 elsif ( $section eq "apache2"     ) { $setup->apache($conf,2)              }
 elsif ( $section eq "apachessl"   ) { $setup->apache($conf,"ssl")          }
 elsif ( $section eq "apacheconf"  ) { $apache->conf_patch($conf)           }
-elsif ( $section eq "ucspi"       ) { $setup->ucspi($conf)                 }
+
+#  Qmail & related
+elsif ( $section eq "ucspi"       ) { $setup->ucspi_tcp($conf)             }
+elsif ( $section eq "daemontools" ) { $setup->daemontools($conf)           }
 elsif ( $section eq "ezmlm"       ) { $setup->ezmlm($conf)                 }
 elsif ( $section eq "vpopmail"    ) { $setup->vpopmail($conf)              }
-elsif ( $section eq "vpeconfig"   ) { $setup->config_vpopmail_etc($conf)   }
+elsif ( $section eq "vpeconfig"   ) { $setup->vpopmail_etc($conf)          }
 elsif ( $section eq "vpopmysql"   ) { $setup->vpopmail_mysql_privs($conf)  }
 elsif ( $section eq "vqadmin"     ) { $setup->vqadmin($conf)               }
 elsif ( $section eq "qmail"       ) { $qmail->install_qmail($conf)         } 
 elsif ( $section eq "qmailconf"   ) { $qmail->config($conf)                } 
 elsif ( $section eq "netqmail"    ) { $qmail->netqmail($conf)              } 
 elsif ( $section eq "netqmailmac" ) { $qmail->netqmail_virgin($conf)       } 
+elsif ( $section eq "djbdns"      ) { $setup->djbdns()                     }
+
+elsif ( $section eq "courier"     ) { $setup->courier($conf)               } 
+elsif ( $section eq "courierconf" ) { $setup->courier_config($conf)        } 
+
+#  Web Mail & Admin interfaces
 elsif ( $section eq "qmailadmin"  ) { $setup->qmailadmin($conf)            }
 elsif ( $section eq "sqwebmail"   ) { $setup->sqwebmail($conf)             } 
-elsif ( $section eq "courier"     ) { $setup->courier($conf)               } 
-elsif ( $section eq "courierconf" ) { $setup->config_courier($conf)        } 
 elsif ( $section eq "squirrelmail") { $setup->squirrelmail($conf)          } 
+
+#  Mail Filtering
 elsif ( $section eq "filter"      ) { $setup->filtering($conf)             }
+elsif ( $section eq "razor"       ) { $setup->razor($conf)                 }
 elsif ( $section eq "maildrop"    ) { $setup->maildrop($conf)              }
 elsif ( $section eq "clamav"      ) { $setup->clamav($conf)                }
 elsif ( $section eq "qmailscanner") { $setup->qmail_scanner($conf)         }
 elsif ( $section eq "simscan"     ) { $setup->simscan($conf)               }
-elsif ( $section eq "qss"         ) { $setup->qs_stats($conf)              }
-elsif ( $section eq "supervise"   ) { $setup->supervise($conf)             }
+elsif ( $section eq "simconf"     ) { $setup->simscan_conf($conf)          }
+elsif ( $section eq "simtest"     ) { $setup->simscan_test($conf)          }
+elsif ( $section eq "spamassassin") { $setup->spamassassin($conf)          }
+
+#  Logs, Statistics & Monitoring
 elsif ( $section eq "maillogs"    ) { $setup->maillogs($conf)              }
-elsif ( $section eq "rrdutil"     ) { $setup->rrdutil($conf)               }
-elsif ( $section eq "mattbundle"  ) { $setup->mattbundle($conf)            }
+elsif ( $section eq "qss"         ) { $setup->qs_stats($conf)              }
 elsif ( $section eq "socklog"     ) { $setup->socklog($conf)               }
-elsif ( $section eq "toaster"     ) { $utility->mailtoaster($debug)        }
-elsif ( $section eq "jailadd"     ) { $freebsd->jail_create()              }
-elsif ( $section eq "jailstart"   ) { $freebsd->jail_start()               }
-elsif ( $section eq "jaildelete"  ) { $freebsd->jail_delete()              }
+elsif ( $section eq "isoqlog"     ) { $setup->isoqlog($conf)               }
+elsif ( $section eq "rrdutil"     ) { $setup->rrdutil($conf)               }
+elsif ( $section eq "supervise"   ) { $setup->supervise($conf)             }
+
+# test targets
 elsif ( $section eq "test"        ) { $setup->test($conf)                  }
+elsif ( $section eq "filtertest"  ) { $setup->filtering_test($conf)        }
+elsif ( $section eq "authtest"    ) { $setup->test_auth($conf)             }
+elsif ( $section eq "imap"        ) { $setup->imap_test_auth($conf)        }
+elsif ( $section eq "pop3"        ) { $setup->pop3_test_auth($conf)        }
+elsif ( $section eq "smtp"        ) { $setup->smtp_test_auth($conf)        }
+elsif ( $section eq "test2"       ) { print "ok\n"                         }
+
+#  misc 
+elsif ( $section eq "mattbundle"  ) { $setup->mattbundle($conf)            }
+elsif ( $section eq "logmonster"  ) { $setup->logmonster($conf)            }
+elsif ( $section eq "mrm"         ) { $setup->mrm($conf)                   }
+elsif ( $section eq "toaster"     ) { $utility->mailtoaster($debug)        }
+elsif ( $section eq "nictool"     ) { $setup->nictool($conf, $debug)       }
 elsif ( $section eq "all"         ) 
 {
 	$setup->dependencies($conf);
@@ -131,7 +167,7 @@ elsif ( $section eq "all"         )
 	$setup->mysqld($conf); 
 	$setup->apache($conf,2);
 	$setup->phpmyadmin($conf);
-	$setup->ucspi($conf);
+	$setup->ucspi_tcp($conf);
 	$setup->ezmlm($conf);
 	$setup->vpopmail($conf);
 	$setup->maildrop($conf);
@@ -161,14 +197,24 @@ sub usage
 
            pre - installs a list of programs and libraries other toaster components need
           perl - installs or upgrades perl
+
+                     FreeBSD Specific
          ports - updates your ports tree, installs the pkg_* tools
        sources - update your FreeBSD sources (/usr/src)
+       jailadd - creates a new jail
+     jailstart - starts up an existing jail
+    jaildelete - deletes an existing jail
+
+                    Standard Daemons & Utilities
          mysql - installs MySQL
     phpmyadmin - installs phpMyAdmin
         apache - installs Apache 
      apachessl - installs self signed SSL certs for Apache
     apacheconf - patches httpd.conf for use with Mail::Toaster
-         ucspi - install ucspi w/MySQL patch
+
+                     Qmail and related tools
+         ucspi - install ucspi-tcp w/MySQL patch
+   daemontools - install daemontools
          ezmlm - install EzMLM idx
       vpopmail - installs vpopmail
      vpeconfig - configure ~vpopmail/etc/tcp.smtp
@@ -178,25 +224,44 @@ sub usage
      qmailconf - configure various qmail control files
       netqmail - installs netqmail 
    netqmailmac - installs netqmail with no patches
+        djbdns - install the djbdns program
+
+       courier - installs courier imap & pop3 daemons
+   courierconf - post install configure for courier
+
+                   Web Mail and Admin interfaces
     qmailadmin - installs qmailadmin
      sqwebmail - installs sqwebmail (webmail app)
-       courier - installs courier imap & pop3 daemons
   squirrelmail - installs squirrelmail (webmail app)
+
+                     Mail Filtering
         filter - installs SpamAssassin, ClamAV, DCC, razor, and more
+         razor - installs the razor2 agents
       maildrop - installs maildrop and mailfilter
         clamav - installs just ClamAV
   qmailscanner - installs Qmail-Scanner & qmailscanner stats
        simscan - install simscan
-           qss - installs qmailscanner stats
-     supervise - creates the directories to be used by svscan
+       simconf - configure simscan 
+       simtest - run email tests to verify that simscan is working
+  spamassassin - install and configure spamassassin
+
+                  Logs, Statistics, and Monitoring
       maillogs - creates the mail logging directories
-       rrdutil - installs rrdutil
-    mattbundle - install MATT::Bundle
-       toaster - install Mail::Toaster
+           qss - installs qmailscanner stats
        socklog - installs socklog
-       jailadd - creates a new jail
-     jailstart - starts up an existing jail
-    jaildelete - deletes an existing jail
+       isoqlog - installs and configured isoqlog
+       rrdutil - installs rrdutil
+     supervise - creates the directories to be used by svscan
+
+          test - runs a complete test suite against your server
+    filtertest - runs the simscan and qmail-scanner email scanner tests
+      authtest - authenticates against pop, imap, and smtp servers
+
+       toaster - install Mail::Toaster
+    mattbundle - install MATT::Bundle
+           mrm - install Mysql::Replication
+    logmonster - install Apache::Logmonster
+       nictool - install nictool (http://www.nictool.com/)
            all - installs everything shown on the toaster INSTALL page
 
 EOUSAGE
@@ -210,68 +275,119 @@ __END__
   toaster_setup.pl -s [ section ] [-debug]
 
            pre - installs a list of programs and libraries other toaster components need
+          perl - installs or upgrades perl
+
+                     FreeBSD Specific
          ports - updates your ports tree, installs the pkg_* tools
        sources - update your FreeBSD sources (/usr/src)
+       jailadd - creates a new jail
+     jailstart - starts up an existing jail
+    jaildelete - deletes an existing jail
+
+                    Standard Daemons & Utilities
          mysql - installs MySQL
     phpmyadmin - installs phpMyAdmin
         apache - installs Apache 
      apachessl - installs self signed SSL certs for Apache
     apacheconf - patches httpd.conf for use with Mail::Toaster
-         ucspi - install ucspi w/MySQL patch
+
+                     Qmail and related tools
+         ucspi - install ucspi-tcp w/MySQL patch
+   daemontools - install daemontools
          ezmlm - install EzMLM idx
       vpopmail - installs vpopmail
      vpeconfig - configure ~vpopmail/etc/tcp.smtp
      vpopmysql - run the vpopmail MySQL grant and db create commands
+       vqadmin - install vqadmin
          qmail - installs qmail with toaster patches
-     qmailconf - configure various /var/qmail/control/* files
+     qmailconf - configure various qmail control files
       netqmail - installs netqmail 
+   netqmailmac - installs netqmail with no patches
+        djbdns - install the djbdns program
+
+       courier - installs courier imap & pop3 daemons
+   courierconf - post install configure for courier
+
+                   Web Mail and Admin interfaces
     qmailadmin - installs qmailadmin
      sqwebmail - installs sqwebmail (webmail app)
-       courier - installs courier imap & pop3 daemons
   squirrelmail - installs squirrelmail (webmail app)
+
+                     Mail Filtering
         filter - installs SpamAssassin, ClamAV, DCC, razor, and more
+         razor - installs the razor2 agents
+      maildrop - installs maildrop and mailfilter
         clamav - installs just ClamAV
   qmailscanner - installs Qmail-Scanner & qmailscanner stats
-           qss - installs qmailscanner stats
-     supervise - creates the directories to be used by svscan
+       simscan - install simscan
+       simconf - configure simscan 
+       simtest - run email tests to verify that simscan is working
+
+                  Logs, Statistics, and Monitoring
       maillogs - creates the mail logging directories
-       rrdutil - installs rrdutil
-    mattbundle - install MATT::Bundle
-       toaster - install Mail::Toaster
+           qss - installs qmailscanner stats
        socklog - installs socklog
+       isoqlog - installs and configured isoqlog
+       rrdutil - installs rrdutil
+     supervise - creates the directories to be used by svscan
+
+          test - runs a complete test suite against your server
+    filtertest - runs the simscan and qmail-scanner email scanner tests
+      authtest - authenticates against pop, imap, and smtp servers
+
+       toaster - install Mail::Toaster
+    mattbundle - install MATT::Bundle
+           mrm - install Mysql::Replication
+    logmonster - install Apache::Logmonster
            all - installs everything shown on the toaster INSTALL page
 
 
 =head1 AUTHOR
 
-	Matt Simerson <matt@tnpi.biz>
+Matt Simerson <matt@tnpi.biz>
 
 =head1 BUGS
 
-	None known. Report any to author.
-
+None known. Report any to author.
 
 =head1 TODO
 
-	Check if daemons are running before installs.
-	Turn entire process into a ./install_it_all script
-	Add -s dnscache section to install a DNS stub resolver
-
+ Add -s dnscache section to install a DNS stub resolver
+ Check if daemons are running before installs. - mostly done
+ Turn entire process into a ./install_it_all script  - done
 
 =head1 SEE ALSO
 
-Mail::Toaster::CGI, Mail::Toaster::DNS, 
-Mail::Toaster::Logs, Mail::Toaster::Qmail, toaster.conf
-Mail::Toaster::Setup, Mail::Toaster::Conf, toaster-watcher.conf
+The following are all man/perldoc pages:
+
+ Mail::Toaster
+ Mail::Toaster::Apache
+ Mail::Toaster::CGI 
+ Mail::Toaster::DNS
+ Mail::Toaster::Darwin
+ Mail::Toaster::Ezmlm
+ Mail::Toaster::FreeBSD
+ Mail::Toaster::Logs
+ Mail::Toaster::Mysql
+ Mail::Toaster::Passwd
+ Mail::Toaster::Perl
+ Mail::Toaster::Provision
+ Mail::Toaster::Qmail
+ Mail::Toaster::Setup
+ Mail::Toaster::Utility
+
+ Mail::Toaster::Conf
+ toaster.conf 
+ toaster-watcher.conf
 
  http://matt.simerson.net/computing/mail/toaster/
+ http://matt.simerson.net/computing/mail/toaster/docs/
  http://matt.simerson.net/computing/mail/toaster/faq.shtml
  http://matt.simerson.net/computing/mail/toaster/changelog.shtml
 
-
 =head1 COPYRIGHT
 
-Copyright (c) 2004, The Network People, Inc.
+Copyright (c) 2004-2005, The Network People, Inc.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
